<?php

namespace Intraservis\Application;

use InvalidArgumentException;

class FlashMessages
{
	private array $session;

	/**
	 * @var array úložisko správ
	 */
	private array $messages = [
		"current" => [],
		"next" => []
	];

	private string $sessionKey;

	private array $types = [
		"success",
		"info",
		"warning",
		"danger",
	];


	public function __construct(?array &$session = null, $sessionKey = "intraservis.flash")
	{
		if ($session === null) {
			$this->session = &$_SESSION;
		} else {
			$this->session = &$session;
		}

		if (isset($this->session[$sessionKey]) === true) {
			$this->messages["current"] = $this->session[$sessionKey];
		}

		$this->sessionKey = $sessionKey;

		register_shutdown_function([$this, 'storeMessages']);
	}


	/**
	 * Pridanie správy
	 *
	 * @param string text správy
	 * @param string typ správy - info, error
	 * @param string požiadavka - current, next
	 * @return void
	 */
	public function addMessage($message, $type = "info", $request = "next", $escape = true)
	{
		if (in_array($type, $this->types) === false) {
			throw new InvalidArgumentException("Unknown flash message type: \"$type\"");
		}

		if ($escape === true) {
			$message = htmlspecialchars($message, ENT_QUOTES);
		}

		$key = sha1($message . $type);

		$this->messages[$request][$key] = [
			"type" => $type,
			"message" => $message
		];
	}


	/**
	 * Vyzdvihnutie správ(y)
	 *
	 * @param string|null požiadavka - current, next, null (všetko)
	 * @param bool        príznak či po vyzdvihnutí správ vyčistiť úložisko správ
	 * @return array
	 */
	public function getMessages($request = "current", $flush = true)
	{
		if ($request === null) {
			$messages = $this->messages;

			if ($flush === true) {
				$this->messages = [
					"current" => [],
					"next" => []
				];
			}
		} else {
			$messages = $this->messages[$request];

			if ($flush === true) {
				$this->messages[$request] = [];
			}
		}

		return $messages;
	}


	public function storeMessages()
	{
		$messages = array_merge($this->messages["current"], $this->messages["next"]);
		$this->session[$this->sessionKey] = $messages;
	}
}
